@@ -1,29 +1,31 @@
+use 5.010001;
 use Math::Complex;
+use Math::Polynomial::Solve qw(:utility);
+use Math::Utils qw(:polynomial :compare);
+
+my $fltcmp = generate_fltcmp(2.5e-7);
+
+sub allzeroes
+{
+	my($p_ref, @xvals) = @_;
+	my @yvals = grep {&$fltcmp($_, 0) != 0} pl_evaluate($p_ref, @xvals);
+	return (scalar @yvals)? 0: 1;
+}
 
 sub sumof
 {
 	my $x = cplx(0,0);
-	foreach (@_){$x += $_};
+	for (@_){$x += $_};
 	return $x;
 }
 
 sub prodof
 {
 	my $x = cplx(1,0);
-	foreach (@_){$x *= $_};
+	for (@_){$x *= $_};
 	return $x;
 }
 
-sub fltcmp
-{
-	my($a, $b) = @_;
-	my($flt) = 0.25/16777216;	# a good enough value for testing.
-
-	return -1 if ($a + $flt < $b);
-	return 1 if ($a - $flt > $b);
-	return 0;
-}
-
 #
 # returns 0 (equal) or 1 (not equal). There's no -1 value, unlike other cmp functions.
 #
@@ -36,15 +38,28 @@ sub polycmp
 
 	return 1 if (scalar @polynomial1 != scalar @polynomial2);
 
-	foreach my $c1 (@polynomial1)
+	for my $c1 (@polynomial1)
 	{
 		my $c2 = shift @polynomial2;
-		return 1 if (fltcmp($c1, $c2) != 0);
+		return 1 if (&$fltcmp($c1, $c2) != 0);
 	}
 
 	return 0;
 }
 
+sub polychain2str
+{
+	my(@chain) = @_;
+	my $str = "";
+	for my $j (0..$#chain)
+	{
+		my @c = @{$chain[$j]};
+		$str .= sprintf("    f%2d: [", $j) . join(", ", @c) . "]\n";
+	}
+	return $str;
+}
+
+
 sub cartesian_format_signed($$@)
 {
 	my($fmt_re, $fmt_im, @numbers) = @_;
@@ -53,7 +68,7 @@ sub cartesian_format_signed($$@)
 	$fmt_re ||= "%.15g";	# Provide a default real format
 	$fmt_im ||= "%.15gi";	# Provide a default im format
 
-	foreach $n (@numbers)
+	for $n (@numbers)
 	{
 		#
 		# Is the number part of the Complex package?
@@ -85,7 +100,7 @@ sub cartesian_format($$@)
 	$fmt_re ||= "%.15g";		# Provide a default real format
 	$fmt_im ||= " + %.15gi";	# Provide a default im format
 
-	foreach $n (@numbers)
+	for $n (@numbers)
 	{
 		#
 		# Is the number part of the Complex package?
@@ -111,7 +126,7 @@ sub rootprint
 {
 	my $i = 0;
 	my $line = "";
-	foreach (@_)
+	for (@_)
 	{
 		$line .= "\tx[$i] = " . cartesian_format(undef, undef, $_) . "\n";
 		$i++;
@@ -122,7 +137,7 @@ sub rootprint
 sub rootformat
 {
 	my @fmtlist;
-	foreach (@_)
+	for (@_)
 	{
 		push @fmtlist, cartesian_format(undef, undef, $_);
 	}
